#!/usr/bin/env python
##
## This file is part of Nolimips, a MIPS simulator with unlimited registers
## Copyright (C) 2003, 2004, 2006 Benoit Perrot <benoit@lrde.epita.fr>
##
## Nolimips is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Nolimips is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##

import sys, os
import string, re

import nolimips
from nolimips import depth, class_id

#### Nolimips' program builder generator -------------------

if __name__ != "__main__" or len(sys.argv) != 2:
  print "Usage: .. SRCDIR"
  sys.exit(1)
srcdir = sys.argv[1]

parser = nolimips.InstructionSetParser()
instructions = parser.parse(sys.stdin)

sys.stdout = file("program_builder.hh.tmp", "w")

print nolimips.license
print nolimips.generated_by_hand
print """#ifndef INST_PROGRAM_BUILDER_HH
# define INST_PROGRAM_BUILDER_HH

# include \"inst/all.hh\"
# include \"inst/int_exp.hh\"
# include \"inst/label_exp.hh\"
# include \"inst/op_exp.hh\"
# include \"inst/program.hh\"

# include \"vm/cpu.hh\"

namespace inst
{

  using vm::Cpu;
  
  class ProgramBuilder
  {
  public:
    ProgramBuilder();

  public:
    void reset();

    void set_nop_after_branch(bool nop_after_branch_p);
    
    Program *get();
    
  public:
    void define_inst_label(const misc::unique_string &id);
    void define_data_label(const misc::unique_string &id);

  public:
    void add_byte(int imm);

    void add_half(int imm);

    void add_word(int imm);
    void add_word(LabelExp *l);

    void add_ascii(const std::string *str);
    void add_asciiz(const std::string *str);

    void add_space (int size);

  public:"""

type_map = {
    "register": "Register *",
    "immediate": "Exp *",
    "label": "Exp *",
    "address": "Register *"
    }

for inst in instructions:
  for syntax in inst.syntaxes:
    if (syntax.alias != "") or (syntax.level == "pseudo"):
      continue
    
    if syntax.level == "native":
      proto = ""
      call = ""
      for attribute in inst.format.attributes:
        proto = proto + attribute.type + " *" + attribute.name + ", "
        if attribute.type == "Register":
          call = call + "*"
        call = call + attribute.name + ", "
      proto = string.rstrip(proto, ", ")
      call = string.rstrip(call, ", ")
      print "    void add_" + inst.opcode + "(" + proto + ")"
      print "    {"
      call = "new " + class_id(inst.opcode) + "(" + call + ")"
      print "      program_->text_section().add_inst (" + call + ");"
      if inst.kind == "branch":
        print "      if (nop_after_branch_p_)"
        print "	program_->text_section()."
        print "	  add_inst (new Sll(Register(Register::general, " + \
              "Cpu::zero), "
        print "			    Register(Register::general, " + \
              "Cpu::zero), "
        print "			    new IntExp(0)));"
      print "    }"
      
    else: # syntax.level == "complex"
      proto = ""
      for token in syntax.tokens:
        proto = proto + type_map[token.kind]
        if token.name:
          proto = proto + token.name
        proto = proto + ", "
      proto = string.rstrip(proto, ", ")
      print "    void\tadd_" + inst.opcode + "(" + proto + ");"
      
print """
  protected:
    Program *program_;
    bool nop_after_branch_p_;
  };

} // namespace inst

# include "inst/program_builder.hxx"

#endif // !INST_PROGRAM_BUILDER_HH"""

## Epilogue
sys.stdout.close()
sys.stdout = sys.__stdout__
nolimips.lazy_overwrite(srcdir + os.sep + "program_builder.hh",
                        "program_builder.hh.tmp")
os.remove("program_builder.hh.tmp")
