//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2003, 2004, 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
#ifndef TASK_TASK_REGISTER_HH
# define TASK_TASK_REGISTER_HH

# include <string>
# include <vector>
# include <set>
# include <map>

namespace task
{

  class Task;

  class TaskRegister
  {
  protected:
    typedef std::map<std::string,
		     Task const*>::const_iterator const_task_iterator;

  private:
    TaskRegister();
  public:
    static TaskRegister &instance();

  public:
    const std::list<Task const*> &enabled_tasks() const;

  public:
    void register_task(const Task& task);
    void enable_task(const Task& task);
    void enable_task(std::string task_name);

  protected:
    const_task_iterator find_task(const std::string &option);
  public:
    char* parse_args (int argc, char *argv[]);

  public:
    void execute (void);
  private:
    void resolve_dependencies(const Task* task,
			      std::set<Task const*>& visited_tasks,
			      std::list<Task const*>& sorted_tasks);
    void resolve_dependencies();

  public:
    void version(std::ostream& ostr);
    void usage(std::ostream& ostr);
    void help(std::ostream& ostr);
    void selection(std::ostream& ostr);

  private:
    std::map<std::string, std::list<Task const*> > modules_;
    std::map<std::string, Task const*> tasks_;
    std::list<Task const*> enabled_tasks_;
  };

} // namespace task

# include "task/task_register.hxx"

#endif // !TASK_TASK_REGISTER_HH
